(** Time-stamp: <modified the 26/06/2008 (at 16:35) by Erwan Jahier> *)

open CompiledData
open UnifyClock

(** [f ids ve ]checks that [ve] is well-clocked (i.e., for node calls,
   it checks that the argument and the parameter clocks are correct),
   and returns a clock profile that contains all the necessary information
   to be able to check.
*)
val f : id_solver -> subst -> val_exp_eff -> clock_eff list * subst


(** [check_res lxm cel cil] check that the expected output clock
   profile of an expression "cil" is compatible with the result clocks
   "cel" of the expression.

   For instance,  for clocks checking the equation 

      (x,y)=toto(a,b);

   it checks that the clock of "(x,y)" (cel) is compatible with the 
   output clock profile of node toto (cil).

   The type [clock_eff_res] is exported to be able to call that
   function outside this module, which is necessary to clock
   equations (an equation is not an expression (i.e., not a
   val_exp_eff).

*)

val check_res : Lxm.t -> subst -> left_eff list -> clock_eff list -> unit
